'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var React__default = /*#__PURE__*/_interopDefaultLegacy(React);

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return;
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

var isBrowser = typeof window !== 'undefined' && typeof window.document !== 'undefined' && typeof window.document.createElement !== 'undefined' && !(process.env.NODE_ENV !== 'production' && process.env.USE_GLOBAL_CONTEXT_TEST_MODE === 'server');

var useIsomorphicLayoutEffect = isBrowser ? React.useLayoutEffect : function (fn) {
  return fn();
};

var error = function error(message) {
  if (process.env.NODE_ENV === 'production') {
    return;
  }
  /* eslint no-console: 0 */


  console.error(message);
};
var warn = function warn(message) {
  if (process.env.NODE_ENV === 'production') {
    return;
  }
  /* eslint no-console: 0 */


  console.warn(message);
};
var log = function log(message) {
  if (process.env.NODE_ENV === 'production') {
    return;
  }
  /* eslint no-console: 0 */


  console.log(message);
};
var devlog = {
  log: log,
  warn: warn,
  error: error
};

// https://github.com/reduxjs/react-redux (MIT LICENSE)
// https://github.com/dai-shi/use-context-selector (MIT LICENSE)

var refEquality = function refEquality(a, b) {
  return a === b;
};

var createUseSelector = function createUseSelector(context, subscription) {
  function useSelector(selector, equalityFunction) {
    var store = React.useContext(context);

    var _useReducer = React.useReducer(function (s) {
      return s + 1;
    }, 0),
        _useReducer2 = _slicedToArray(_useReducer, 2),
        forceRender = _useReducer2[1];

    var selectedStore;
    var latestSelector = React.useRef();
    var latestStore = React.useRef();
    var latestSelectedStore = React.useRef();
    selectedStore = selector(_objectSpread2({}, store));
    useIsomorphicLayoutEffect(function () {
      latestSelector.current = selector;
      latestStore.current = store;
      latestSelectedStore.current = selectedStore;
    });
    useIsomorphicLayoutEffect(function () {
      var refresh = function refresh(nextStore) {
        if (latestSelector.current === undefined || latestSelectedStore.current === undefined) {
          return;
        }

        try {
          var newSelectedStore = latestSelector.current(_objectSpread2({}, nextStore));
          var isEqualityFunction = equalityFunction || refEquality;

          if (isEqualityFunction(newSelectedStore, latestSelectedStore.current)) {
            return;
          }

          latestSelectedStore.current = newSelectedStore;
        } catch (err) {
          devlog.error(err);
        }

        forceRender();
      };

      subscription.addListener(refresh);
      return function () {
        subscription.deleteListener(refresh);
      };
    }, [context, subscription]);
    return selectedStore;
  }

  return useSelector;
};

var createStore = function createStore(context, subscription) {
  var useGlobalContext = createUseSelector(context, subscription);
  return useGlobalContext;
};

var createBaseContext = function createBaseContext() {
  var context = /*#__PURE__*/React.createContext(null, function () {
    return 0;
  });
  context.displayName = 'GlobalContext';
  return context;
};

var createSubscription = function createSubscription() {
  var listeners = new Set();

  var trySubscribe = function trySubscribe(value) {
    listeners.forEach(function (listener) {
      listener(value);
    });
  };

  var addListener = function addListener(listener) {
    listeners.add(listener);
  };

  var deleteListener = function deleteListener(listener) {
    listeners["delete"](listener);
  };

  var subscription = {
    trySubscribe: trySubscribe,
    addListener: addListener,
    deleteListener: deleteListener
  };
  return subscription;
};

var isFunction = function isFunction(value) {
  return value && {}.toString.call(value) === '[object Function]';
};

var entries = function entries(values) {
  return Object.entries(values);
};

var createUseServerSideDispatch = function createUseServerSideDispatch(contextValueRef, partial, subscription, store) {
  /* eslint no-param-reassign: 0 */
  function useServerSideDispatch(state) {
    var currentState = contextValueRef.current.state[partial];
    var newState = isFunction(state) ? state(currentState) : state;
    contextValueRef.current.state[partial] = newState;

    if (store) {
      store.setState(partial, newState);
    }

    subscription.trySubscribe(contextValueRef.current);
  }

  return useServerSideDispatch;
};

var createContext = function createContext(contextSource) {
  var context = createBaseContext();
  var subscription = createSubscription();
  var useGlobalContext = createStore(context, subscription);
  var Provider = context.Provider;

  var contextProvider = function contextProvider(_ref) {
    var _store$getState;

    var children = _ref.children,
        store = _ref.store;
    var contextValueRef = React.useRef({
      state: {},
      dispatch: {}
    });
    var storeState = (_store$getState = store === null || store === void 0 ? void 0 : store.getState()) !== null && _store$getState !== void 0 ? _store$getState : undefined;
    entries(contextSource).forEach(function (_ref2) {
      var _ref3 = _slicedToArray(_ref2, 2),
          partial = _ref3[0],
          initialState = _ref3[1];

      var initialValue = storeState && storeState[partial] !== undefined ? storeState[partial] : initialState;

      var _useState = React.useState(initialValue),
          _useState2 = _slicedToArray(_useState, 2),
          state = _useState2[0],
          dispatch = _useState2[1];

      contextValueRef.current.state[partial] = state;
      contextValueRef.current.dispatch[partial] = dispatch;

      if (store) {
        store.setState(partial, state);
      }
    });
    React.useEffect(function () {
      subscription.trySubscribe(contextValueRef.current);
    });
    return /*#__PURE__*/React__default['default'].createElement(Provider, {
      value: contextValueRef.current
    }, children);
  };

  var contextServerSideProvider = function contextServerSideProvider(_ref4) {
    var _store$getState2;

    var children = _ref4.children,
        store = _ref4.store;
    var contextValueRef = React.useRef({
      state: {},
      dispatch: {}
    });
    var storeState = (_store$getState2 = store === null || store === void 0 ? void 0 : store.getState()) !== null && _store$getState2 !== void 0 ? _store$getState2 : undefined;
    entries(contextSource).forEach(function (_ref5) {
      var _ref6 = _slicedToArray(_ref5, 2),
          partial = _ref6[0],
          initialState = _ref6[1];

      var state = storeState && storeState[partial] !== undefined ? storeState[partial] : initialState;
      var dispatch = createUseServerSideDispatch(contextValueRef, partial, subscription, store);
      contextValueRef.current.state[partial] = state;
      contextValueRef.current.dispatch[partial] = dispatch;

      if (store) {
        store.setState(partial, state);
      }
    });
    return /*#__PURE__*/React__default['default'].createElement(Provider, {
      value: contextValueRef.current
    }, children);
  };

  return {
    useGlobalContext: useGlobalContext,
    contextProvider: isBrowser ? contextProvider : contextServerSideProvider
  };
};

/**
 * Use "useState" to create a useGlobalContext with global context.
 */
var createUseStateContext = function createUseStateContext(contextSource) {
  var _createContext = createContext(contextSource),
      useGlobalContext = _createContext.useGlobalContext,
      contextProvider = _createContext.contextProvider;

  return [useGlobalContext, contextProvider];
};

var createStore$1 = function createStore(context, subscription) {
  var useGlobalContext = createUseSelector(context, subscription);
  return useGlobalContext;
};

var createUseServerSideDispatch$1 = function createUseServerSideDispatch(contextValueRef, partial, reducer, subscription, store) {
  var useServerSideDispatch = function useServerSideDispatch(action) {
    /* eslint no-param-reassign: 0 */
    var currentState = contextValueRef.current.state[partial];
    var newState = reducer.length === 1 ? reducer(currentState, undefined) : reducer(currentState, action);
    contextValueRef.current.state[partial] = newState;

    if (store) {
      store.setState(partial, newState);
    }

    subscription.trySubscribe(contextValueRef.current);
  };

  return useServerSideDispatch;
};

var createContext$1 = function createContext(contextSource) {
  var context = createBaseContext();
  var subscription = createSubscription();
  var useGlobalContext = createStore$1(context, subscription);
  var Provider = context.Provider;

  var contextProvider = function contextProvider(_ref) {
    var _store$getState;

    var children = _ref.children,
        store = _ref.store;
    var contextValueRef = React.useRef({
      state: {},
      dispatch: {}
    });
    var storeState = (_store$getState = store === null || store === void 0 ? void 0 : store.getState()) !== null && _store$getState !== void 0 ? _store$getState : undefined;
    entries(contextSource).forEach(function (_ref2) {
      var _ref3 = _slicedToArray(_ref2, 2),
          partial = _ref3[0],
          source = _ref3[1];

      var reducer = source.reducer,
          initialState = source.initialState,
          initializer = source.initializer;
      var initialValue = storeState && storeState[partial] !== undefined ? storeState[partial] : initialState;

      var _useReducer = React.useReducer(reducer, initialValue, initializer),
          _useReducer2 = _slicedToArray(_useReducer, 2),
          state = _useReducer2[0],
          dispatch = _useReducer2[1];

      contextValueRef.current.state[partial] = state;
      contextValueRef.current.dispatch[partial] = dispatch;

      if (store) {
        store.setState(partial, state);
      }
    }, {});
    React.useEffect(function () {
      subscription.trySubscribe(contextValueRef.current);
    });
    return /*#__PURE__*/React__default['default'].createElement(Provider, {
      value: contextValueRef.current
    }, children);
  };

  var contextServerSideProvider = function contextServerSideProvider(_ref4) {
    var _store$getState2;

    var children = _ref4.children,
        store = _ref4.store;
    var contextValueRef = React.useRef({
      state: {},
      dispatch: {}
    });
    var storeState = (_store$getState2 = store === null || store === void 0 ? void 0 : store.getState()) !== null && _store$getState2 !== void 0 ? _store$getState2 : undefined;
    entries(contextSource).forEach(function (_ref5) {
      var _ref6 = _slicedToArray(_ref5, 2),
          partial = _ref6[0],
          initialState = _ref6[1].initialState;

      var state = storeState && storeState[partial] !== undefined ? storeState[partial] : initialState;
      var dispatch = createUseServerSideDispatch$1(contextValueRef, partial, contextSource[partial].reducer, subscription, store);
      contextValueRef.current.state[partial] = state;
      contextValueRef.current.dispatch[partial] = dispatch;

      if (store) {
        store.setState(partial, state);
      }
    }, {});
    useIsomorphicLayoutEffect(function () {
      subscription.trySubscribe(contextValueRef.current);
    });
    return /*#__PURE__*/React__default['default'].createElement(Provider, {
      value: contextValueRef.current
    }, children);
  };

  return {
    useGlobalContext: useGlobalContext,
    contextProvider: isBrowser ? contextProvider : contextServerSideProvider
  };
};

/**
 * Use "useReducer" to create a useGlobalContext with global context.
 */
var createUseReducerContext = function createUseReducerContext(contextSource) {
  var _createContext = createContext$1(contextSource),
      useGlobalContext = _createContext.useGlobalContext,
      contextProvider = _createContext.contextProvider;

  return [useGlobalContext, contextProvider];
};

var createStore$2 = function createStore(initialValue) {
  var currentState = _objectSpread2({}, initialValue);

  var getState = function getState() {
    return currentState;
  };

  var setState = function setState(key, value) {
    currentState[key] = value;
  };

  return {
    getState: getState,
    setState: setState
  };
};

exports.createStore = createStore$2;
exports.createUseReducerContext = createUseReducerContext;
exports.createUseStateContext = createUseStateContext;
